/*
Program to replicate the SIC to NAICS bridge of the 1997 Economic Census
Author: Teresa Fort
Created: May 2014 
Last Modified: May 2014
The original data is downloaded from http://www.census.gov/epcd/ec97brdg/index.html (accessed Mar. 23. 2013)
*/

cd "C:\Users\F000KTQ\Dropbox\Research\FK_NAICS\Data"


*Import the raw data
insheet using SIC_to_NAICS.txt, comma clear

set more off

drop v1

*Create SIC and NAICS description variables
*********************************************
rename v2 description

gen SIC=description if v4==.

sort v3 v4

forvalues i=1/15 {
  replace SIC = SIC[_n+`i'] if SIC ==""
  }

rename SIC SIC_description
rename description NAICS_description
*********************************************

*Rename variables
*********************************************
rename v10 Estabs
rename v12 Shipments
rename v14 Payroll
rename v16 Employment
rename v15 Data_suppressed
rename v4 NAICS_code
rename v5 pt
rename v3 SIC_code
rename v6 taxable
*********************************************

*Reorder variables
*********************************************
placevar Employment, before(Data_suppressed)
placevar SIC_code SIC_description, before(NAICS_description)
placevar NAICS_code, before(NAICS_description)

/*
*Drop variables
drop v17
drop v13
drop v11
drop v9
drop v7
drop v8
*/

*Label variables
*********************************************
label variable pt "pt code; if no pt code, the entirety of the NAICS falls within the SIC"
label variable SIC_code "the SIC industry"
label variable NAICS_code "part of the NAICS classified under the SIC"
label variable taxable "NAICS industries classified into taxable or exempt"
label variable Estabs "number of establishments"
label variable Payroll "annual payroll in thousand dollars"
label variable Employment "number of paid employees"
label variable Shipments "Value of shipments, in thousand dollars"
label variable Data_suppressed "D if data suppressed"
*********************************************


*Fix variables
*********************************************
replace taxable = "taxable" if taxable == "t" | taxable=="T" 
replace taxable="exempt" if taxable =="n" | taxable =="N"

replace NAICS_description = "" if length(SIC_code) ==4
*********************************************

***NO IDEA WHAT THIS IS IN THE TEXT FILE--NOT EVIDENT IN THE ONLINE DATASET
drop if v7==10  /*Unclear what v7 is, but v7=0 seems to match website  whereas v7=10 does not.  Represents duplicate entries.  Need some way to check this step*/


**Create SIC total datasets (for each level of aggregation)
*********************************************
save temp, replace

forvalues i=1/5 {
  keep if  NAICS_code == . & length(SIC_code) == `i'	
  keep SIC_code Estabs Shipments Employment Payroll taxable SIC_descr
  gen SIC`i'=substr(SIC_code,1,`i')
  drop SIC_code
  rename SIC_description SIC`i'_description
   if `i'==2 {
    contract SIC2 Estabs Shipments Employment Payroll taxable SIC2_descr
	drop _freq
	drop if SIC2_description=="Transportation, communications, and utilities - Auxiliaries"
	}
  foreach var in Estabs Shipments Employment Payroll {
    rename `var' SIC`i'_`var'
    }
 
  save SIC`i'_tots, replace
  use temp, clear
  }
*********************************************


*Modify dataset
*********************************************
drop if NAICS_code == . & length(SIC_code) == 5	/* Drop the overall SIC4 industry tots */
drop v9

keep if NAICS_code~=.
sort SIC_code NAICS_code

*For SIC-NAICS mappings in which there are break-outs by NAICS description (but the NAICS code is the same) need to drop all the irrelevant obs
save temp, replace
keep if pt==" s"
contract SIC_code NAICS_code taxable
drop _freq 
merge 1:m SIC_code NAICS_code taxable using temp
keep if _merge==2 | (_merge==3 & pt==" s")

/*
Investigate the weird concordance issues
**Looks like the partial sic bridge code is captured in Pt.  Looks like the SIC bridge does not map to multiple NAICS--but need to check systematically

*Pt is sometimes populated, even if there are not multiple sic bridge codes

*If Pt has an s, this is the major NAICS category with the correct naics description that should be kept (could check this at end using a NAICS97 list)
*If Naics is unique within SIC, then it should be kept.

save temp, replace
keep if pt==" s" 
keep SIC_code
contract SIC_code
merge 1:m SIC_code using temp
keep if _merge==3
*/


*********************************************

**Check that each SIC maps to each NAICS once
gen count=1
bysort SIC_code NAICS_code: egen check=sum(count)
summ check
*Only multiple mappings seem to be taxable versus exempt


**Create fraction of Estabs, Employment and Shipments that is allocated to each NAICS code

*1 Fill in missing employment, shipments and payroll
    *Impute based on estabs
	
	*Add in sector tots
	forvalues i=1/5 {
	 if `i'~=2 {
	  drop _merge
	  gen SIC`i'=substr(SIC_code,1,`i')
	  merge m:1 SIC`i' taxable using SIC`i'_tots
	  }
	  }
drop if SIC_code==""
	  
	  
*2 Calculate total SIC employment 

 /*Ideally, one would calculate total unknown employment by subtracting total known employment from all employment. 
   Unfortunately, this doesn't work because adding the totals at the 5 digit SIC (4 digits plus letter) does not always equal the aggregate amount stated
   Below I tried to investigate why.  It seems to be the case that there are mistakes/inconsistencies in the tables.
   For example, SIC code 1799 says it has 525,751 employees, but when you add up the three digit totals, you get 539,755
   This leads to negative values for unknown employment in some cases.  Clearly nonsense.
   Even more bizarrely, I don't see this problem for the number of Estabs  
   Note that is generally the case that the numbers seem to be rounded slightly differently.
   
  gen imputed=0
  gen temp=Estabs if Data_suppressed=="D"
  foreach var in Employment {
     foreach i in 5 4 3 1 {
	 bysort SIC`i': egen SIC`i'_`var'_known=sum(`var') /* Sum employment that is not supressed for each level of aggregation */
	 gen SIC`i'_`var'_unknwn=SIC`i'_`var'-SIC`i'_`var'_known if SIC`i'_`var'~=0   /* Create unknown total employment for each level of aggregation */
	 bysort SIC`i': egen SIC`i'_Estabs_unknwn_`var'=sum(temp) 
     gen avg`i'_`var'_old=SIC`i'_`var'/SIC`i'_Estabs  /*Calculate average var per establishment */
	 gen avg`i'_`var'=SIC`i'_`var'_unknwn/SIC`i'_Estabs_unknwn_`var'  /*Calculate average var per establishment */
	     replace imputed=`i' if Estabs~=0 & (`var'==0 | `var'==.)
	     replace `var'=Estabs*avg`i'_`var' if Estabs~=0 & (`var'==0 | `var'==.) /* Impute var */
	 }
	 }

	 bysort SIC1: egen test=sum(Employment)
	 bysort SIC3: egen test3=sum(Employment)
	 gen chc3=SIC3_Employment-test3
	 
	 bysort SIC4: egen test4=sum(Employment)
	 
	 bysort SIC4: egen test5=sum(Estabs)
	 NEGATIVE UNKNOWNs getting created
	 
	 *Check Employment totals
	 bysort SIC4: egen check1=sum(Employment)
	 gen check2=SIC4_Employment-check1 if SIC4_Emp~=.
	 summ check2 if SIC4_Employment~=0
	 browse check* SIC4_Employment Employment if Data_sup=="D"
	 stop
	*/ 
	
	**Instead, use the stated amount of employment and estabs at higher levels of aggregation to impute.
	 gen imputed=0
  
  foreach var in Employment Shipments Payroll {
     foreach i in 5 4 3 1 { 
     gen avg`i'_`var'=SIC`i'_`var'/SIC`i'_Estabs  /*Calculate average var per establishment */	 
	     replace imputed=`i' if Estabs~=0 & (`var'==0 | `var'==.)
	     replace `var'=Estabs*avg`i'_`var' if Estabs~=0 & (`var'==0 | `var'==.) /* Impute var */
	 }
	 }
	 
  label variable imputed "Vars imputed based on estabs and the average fraction of var/estabs at i level"
   
keep SIC_code NAICS_code Estabs Employment Shipments Payroll SIC_description NAICS_description taxable imputed


**Make Sector variables
gen sic_var=substr(SIC_code,1,1)
replace sic_var=substr(SIC_code,2,1) if sic_var=="L"

gen SIC_sector="Mining" if sic_var=="B"
replace SIC_sector="Construction" if sic_var=="C"
replace SIC_sector="Manufacturing" if sic_var=="D"
replace SIC_sector="Transportation, communications, utilities" if sic_var=="E"
replace SIC_sector="Wholesale" if sic_var=="F"
replace SIC_sector="Retail" if sic_var=="G"
replace SIC_sector="FIRE" if sic_var=="H"
replace SIC_sector="Services" if sic_var=="I"
drop sic_var


tostring NAICS_code, replace
gen NAICS_sector=substr(NAICS_code,1,2)
destring NAICS_sector, replace
gen NAICS_sector_description =""
replace NAICS_sector_description = "Mining" if NAICS_sector ==21
replace NAICS_sector_description = "Construction" if NAICS_sector ==23
replace NAICS_sector_description = "Manufacturing" if NAICS_sector>=31 & NAICS_sector<=33
replace NAICS_sector_description = "Transportation, communications, utilities" if NAICS_sector ==48 |NAICS_sector ==49 | NAICS_sector ==22
replace NAICS_sector_description = "Wholesale" if NAICS_sector ==42
replace NAICS_sector_description = "Retail" if NAICS_sector ==44| NAICS_sector ==45
replace NAICS_sector_description = "FIRE" if NAICS_sector == 52 | NAICS_sector ==53
replace NAICS_sector_description = "Services" if NAICS_sector ==51
replace NAICS_sector_description = "Services" if NAICS_sector >=54
drop NAICS_sector
rename NAICS_sector_description NAICS_sector

**Clean up SIC_codes and deal with Auxiliaries
replace SIC_code=substr(SIC_code,2,4)
replace SIC_code="AUX" if substr(SIC_code,2,2)==""



***Save final dataset
save SIC_NAICS_concordance.dta, replace

